
#include <avr/io.h>
#include <avr/interrupt.h>
#include <util/delay.h>
#include <avr/pgmspace.h>
#include <stdio.h>
#include <string.h>
#include "lcd.h"

#define BTN_UP 		(1 << PC2)
#define BTN_LEFT 	(1 << PC1)
#define BTN_DOWN	(1 << PC0)
#define BTN_RIGHT	(1 << PD6)

#define BTN_UP_PRESSED		((PINC & BTN_UP) == 0)
#define BTN_LEFT_PRESSED	((PINC & BTN_LEFT) == 0)
#define BTN_DOWN_PRESSED	((PINC & BTN_DOWN) == 0)
#define BTN_RIGHT_PRESSED	((PIND & BTN_RIGHT) == 0)

typedef enum {
	RIGHT = 1,
	LEFT = 2,
	UP = 3,
	DOWN = 4
} direction;

#define EASY		3
#define HARD		4

#define LCD_HT 	6
#define LCD_WH  84

#define MAX_LEN 150
#define INITIAL_X	44
#define INITIAL_Y	20

direction curr_direction = RIGHT;
unsigned char game_mode = EASY;
unsigned char pixels[LCD_WH][LCD_HT] = {0};
int score = 0;
int length = 0;
unsigned char head1_x = 0, tail2_x = 0, head1_y = 0, tail2_y = 0;
unsigned char body_x[MAX_LEN + 4] = {0};
unsigned char body_y[MAX_LEN + 4] = {0};
unsigned char food_x;
unsigned char food_y;
int speed = 200;
int counter = 0;

void draw_main_menu()
{
	unsigned char y_arrow = 3;
	while (1) {
		lcd_goto_xy_exact(15, 1);
		lcd_str("Difficulty");
		lcd_goto_xy_exact(15, 3);
		lcd_str("Easy");
		lcd_goto_xy_exact(15, 4);
		lcd_str("Hard");
		
		lcd_goto_xy_exact(5, y_arrow);
		lcd_chr('-');
		_delay_ms(100);
		lcd_clear();
		if (BTN_DOWN_PRESSED) {
			if (y_arrow == 4)
				y_arrow = 3;
			else
				y_arrow++;
		}
		
		if (BTN_UP_PRESSED) {
			if (y_arrow == 3)
				y_arrow = 4;
			else
				y_arrow--;
		}
		
		if (BTN_RIGHT_PRESSED) {
			game_mode = y_arrow;
			if (game_mode == HARD)
				speed = 100;
			break;
		}
	}
}

void draw_pixel(unsigned char x, unsigned char y, unsigned char value)
{
	lcd_goto_xy_exact(x, y);
	lcd_base_addr();
	lcd_send(0xFF, LCD_DATA);
	pixels[x][y] = value;
}

void set_pixel(unsigned char x, unsigned char y, unsigned char value) {
	char poz_y_bloc = (y-1) / 8 + 1;
	char poz_y_bit = (y-1) % 8;
	if (value == 1)
		pixels[x-1][poz_y_bloc-1] |= 1 << poz_y_bit;
	else
		pixels[x-1][poz_y_bloc-1] &= ~(1 << poz_y_bit);
	lcd_goto_xy_exact(x, poz_y_bloc);
	lcd_base_addr();
	lcd_send(pixels[x-1][poz_y_bloc-1], LCD_DATA);
}

void create_food()
{
	int ok = 0, i, j;

	while (!ok) {
		ok = 1;
		food_x = rand() % 80 + 1;
		food_y = rand() % 40 + 3;
		
	}
	
	set_pixel(food_x, food_y, 1);
	set_pixel(food_x+1, food_y, 1);
	set_pixel(food_x, food_y+1, 1);
	set_pixel(food_x+1, food_y+1, 1);
}

void draw_frame()
{
	int i;
	lcd_goto_xy_exact(1, 1);
	lcd_base_addr();
	for (i = 1; i <= LCD_WH; ++i) {
		lcd_send(0x01, LCD_DATA);
		pixels[i-1][0] = 1;
	}
	
	lcd_goto_xy_exact(1, LCD_HT);
	lcd_base_addr();
	for (i = 1; i <= LCD_WH; ++i) {
		lcd_send(0x80, LCD_DATA);
		pixels[i-1][5] = 128;
	}

	for (i = 1 ; i <= LCD_HT ; i++) {
		draw_pixel(1, i, 255);
		draw_pixel(LCD_WH, i, 255);
	}

	return;
	
	
}

void move(direction d)
{
	int i;
	if (d == RIGHT)
		head1_x++;
	else if (d == LEFT)
		head1_x--;
	else if (d == UP)
		head1_y--;
	else if (d == DOWN)
		head1_y++;
	set_pixel(head1_x, head1_y, 1);
	set_pixel(tail2_x, tail2_y, 0);
	
	
	if (head1_x == food_x && head1_y == food_y || head1_x == food_x + 1 && head1_y == food_y || 
head1_x == food_x && head1_y  == food_y +1 || head1_x == food_x + 1 && head1_y == food_y + 1 ) {
		score++;
		body_x[length] = head1_x;
		body_y[length] = head1_y;
		length++;
		create_food();
		create_food();
		create_food();
	} else {
		for (i = 1 ; i < length ; i++) {
			body_x[i-1] = body_x[i];
			body_y[i-1] = body_y[i];
		}
		body_x[length-1] = head1_x;
		body_y[length-1] = head1_y;
		tail2_x = body_x[0];
		tail2_y = body_y[0];
	}
}

void draw_snake()
{
	int i = 0, ok = 1, j;
	for (i = 0; i < 4; ++i) {
		length++;
		body_x[3-i] = INITIAL_X - i;
		body_y[3-i] = INITIAL_Y;
		set_pixel(body_x[3-i], body_y[3-i], 1);
	}
	
	head1_x = body_x[length-1];
	head1_y = body_y[length-1];
	
	tail2_x = body_x[0];
	tail2_y = body_y[0];
	
	while (1) {
		if (BTN_UP_PRESSED && curr_direction != DOWN)
			curr_direction = UP;
		if (BTN_DOWN_PRESSED && curr_direction != UP)
			curr_direction = DOWN;
		if (BTN_LEFT_PRESSED && curr_direction != RIGHT)
			curr_direction = LEFT;
		if (BTN_RIGHT_PRESSED && curr_direction != LEFT)
			curr_direction = RIGHT;
			
		move(curr_direction);
		for (i = 0; i < length-1 && ok; i++) {
			if ((body_x[i] == head1_x) && (body_y[i] == head1_y)) 
				ok = 0;
		}
		
		if ((head1_x == 1) || (head1_y == 1) || (head1_x == 84) || (head1_y == 48))
			ok = 0;


		if (!ok)
			break;
		_delay_ms(speed);
		counter += speed;
		
	}
}

void last_screen()
{
	lcd_clear();
	lcd_goto_xy_exact(20, 3);
	char buff[4];
	memset(buff, 0, 4);
	score += 256;
	sprintf(buff, "%d", score);
	lcd_str("Your score: ");
	lcd_str(buff);
	_delay_ms(3000);
	lcd_clear();
	while (1) {
		lcd_goto_xy_exact(12, 3);
		lcd_str("PRESS RESET");
		
	}
}

int main(void)
{
	
	lcd_init();
	lcd_contrast(0x40);

	DDRC &= ~(1 << PC0);
	DDRC &= ~(1 << PC1);
	DDRC &= ~(1 << PC2);
	DDRD &= ~(1 << PD6);
	PORTC |= (1 << PC0);
	PORTC |= (1 << PC1);
	PORTC |= (1 << PC2);
	PORTD |= (1 << PD6);

	while (1) {
		
		draw_main_menu();
		draw_frame();
		create_food();
		draw_snake();
		last_screen();
		_delay_ms(500);
		curr_direction = RIGHT;
		game_mode = EASY;
		score = 0;
		length = 0;
		head1_x = 0; tail2_x = 0; head1_y = 0; tail2_y = 0;
		food_x = 0;
		food_y = 0;
		speed = 200;
		counter = 0;
	}
}
